//example of generating sin wave with timer PWM by DMA and measuring it by ADC with timer triggering(sending by uart to PC)
//display data on PC with SerialPlot https://hackaday.io/project/5334-serialplot-realtime-plotting-software

#define led          GPIOC_ODR.B13
#define BUF_DEPTH    1000
#define SIN_STEPS    120

//sin table generated by Smart Sine http://tahmidmc.blogspot.com/2012/10/smart-sine-software-to-generate-sine.html
const unsigned int sinTable[SIN_STEPS] =
{500, 526, 552, 578, 604, 629, 655, 679, 703, 727, 750, 772, 794, 815, 835, 854, 872, 889, 905, 919, 933, 946, 957, 967, 976, 983, 989, 994, 997, 999, 1000, 999, 997, 994, 989, 983, 976, 967, 957, 946, 933, 919, 905, 889, 872, 854, 835, 815, 794, 772, 750, 727, 703, 679, 655, 629, 604, 578, 552, 526, 500, 474, 448, 422, 396, 371, 345, 321, 297, 273, 250, 228, 206, 185, 165, 146, 128, 111, 95, 81, 67, 54, 43, 33, 24, 17, 11, 6, 3, 1, 0, 1, 3, 6, 11, 17, 24, 33, 43, 54, 67, 81, 95, 111, 128, 146, 165, 185, 206, 228, 250, 273, 297, 321, 345, 371, 396, 422, 448, 474};

unsigned int data_int[BUF_DEPTH];
unsigned int num=0;
char data_send_time=0;

 void adc_inter() iv IVT_INT_ADC1_2 // adc interrupt after EOC
 {
     ADC1_SRbits.EOC=0;
     
     if(data_send_time==0)// change buffer only if it is not used
     {
         led=0;
         if(num<BUF_DEPTH)// collect buffer
         {
             data_int[num]=ADC1_DRbits.DATA;
             num++;
         }
         else // buffer is full and time to send it to PC
         {
             num=0;
             data_send_time=1;
         }
         led=1;
     }
 }

void timer_3_init()// 1000Hz
 {
       RCC_APB1ENR.TIM3EN = 1;       // Enable clock gating for timer module
       TIM3_CR1.CEN = 0;             // Disable timer
       TIM3_PSC = 7199;              // Set timer prescaler.
       TIM3_ARR = 9;                //
       TIM3_CR2bits.MMS=2;           // The update event is selected as trigger output (TRGO)
       TIM3_CR1.CEN = 1;             // Enable timer
 }

 void ADC_init(void)
 {
        GPIO_Analog_Input(&GPIOA_BASE, _GPIO_PINMASK_0);// channel 0

        RCC_APB2ENRbits.ADC1EN=1;       //enable clock on ADC1
        ADC1_CR2bits.EXTSEL=4;          //Timer 3 TRGO event
        ADC1_CR2bits.EXTTRIG=1;         //External trigger for ADC on
        ADC1_SMPR2bits.SMP0=1;          //Sample time 7.5 cycles
        ADC1_CR1bits.EOCIE=1;           //Interrupt enable for EOC
        ADC1_CR2bits.ADON=1;            //ADC enable
        NVIC_IntEnable(IVT_INT_ADC1_2); //Interrupt enable for ADC1_2
 }
 
 //sin_real_freq=FCPU/((TIM4_PSC+1)*(TIM4_ARR+1)*SIN_STEPS)
 //sin_real_freq(here)=72000000/((19+1)*(999+1)*120)=30 Hz
 
 void init_TIM4()// init timer for PWM by table
{
        RCC_APB1ENRbits.TIM4EN=1;      //enable clock on timer 4
        TIM4_CCERbits.CC3E=1;          //CC4 channel configured as output
        TIM4_CCMR2_Outputbits.OC3M=6;  //Output compare 3 mode (PWM)
        TIM4_CCMR2_Outputbits.OC3PE=1; //Output compare 3 preload enable
        TIM4_DIERbits.CC3DE=1;         //Capture/Compare 3 DMA request enable
        TIM4_CR1bits.CEN=1;            //Counter enable
        TIM4_CR1bits.ARPE=1;           //Auto-reload preload enable
        TIM4_PSC=19;                   //Prescaler value
        TIM4_ARR=999;                 //max count
        TIM4_CCR3=500;                //CCR3 is the value to be loaded in the actual capture/compare 3 register (preload value).
}

void init_DMA()// init sending data from table to timer without CPU  5ch
{
        RCC_AHBENRbits.DMA1EN=1; //DMA1 clock enable
        DMA1_CPAR5=&TIM4_CCR3;   //DMA1 channel 5 peripheral address register(set address of TIM4_CCR3)
        DMA1_CMAR5=&sinTable;    //DMA1 channel x memory address register(set table address)
        DMA1_CNDTR5=SIN_STEPS;   //DMA channel x number of data register
        DMA1_CCR5=0x000005B1;    //EN=1(Channel enable),DIR=1(Data transfer direction:Read from memory),CIRC=1(Circular mode enabled),MINC=1(Memory increment mode enabled),PSIZE=MSIZE=01(16-bits)
}

void init_GPIO()// init gpio to work with timer
{
        RCC_APB2ENRbits.IOPBEN=1; //IO port B clock enable
        RCC_APB2ENRbits.AFIOEN=1; //Alternate function IO clock enable
        GPIOB_CRHbits.MODE8=1;    //10: Alternate function output Push-pull
        GPIOB_CRHbits.CNF8=2;     //Output mode, max speed 2 MHz
}

void init_PWM_sin()
{
        init_GPIO();
        init_TIM4();
        init_DMA();
}

void init_peripherals(void)
{
     GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_13); //led
     GPIOC_ODR.B13=1;//off
     
     ADC_init();
     timer_3_init();
     
     init_PWM_sin();
     
     UART1_Init(115200);
}

void adc_buf_send()
{
        unsigned int i=0;
        int tempi=0;
        char temp=0;

        for(i=0;i<BUF_DEPTH;i++)   // put data byte after byte
        {
            tempi=data_int[i];
            temp=tempi>>8;
            UART1_Write(temp);
            temp=tempi&0x00FF;
            UART1_Write(temp);
            //Delay_ms(1);
        }

}

void from_adc_to_milivolts(void)
{
        unsigned int i=0;
        unsigned long temp=0;
        for(i=0;i<BUF_DEPTH;i++)   //
        {
             temp=data_int[i];
             temp=temp*3335;//vcc in milivolts
             temp=temp/4095;
             data_int[i]=temp;
        }
}

void main()//
{
      init_peripherals();
      
      while(1)
      {
           if(data_send_time==1)//
           {
               from_adc_to_milivolts();
               adc_buf_send();
               data_send_time=0;//start to collect new buffer
           }
      }
}